@@ -19,7 +19,7 @@ our @EXPORT_OK= qw( is_port_open
                     get_ranges
                     get_option_file_contents ) ;
 
-our $VERSION="2.0.99b";
+our $VERSION="3.0.00";
 our $DEBUG;
 
 BEGIN {
@@ -485,7 +485,7 @@ You can create a group of sandboxes without any replication among its members.
 If you need three servers of the same version, you can use
 
  $ make_multiple_sandbox /path/to/tarball 
-    
+
 If you need servers of different versions in the same group, you may like
 
  $ make_multiple_custom_sandbox /path/to/tarball1 path/to/tarball2 /path/to/tb3 
@@ -517,6 +517,17 @@ or
 
 If you call this program several times from the same directory, it will check if the compiled binaries are newer than the extracted ones, and if they aren't, it will reuse the ones created during the previous run, thus saving time and CPU.
 
+=head2 Creating a sandbox from already installed binaries
+
+The script C<make_sandbox_from_installed> tries to create a sandbox using already installed binaries.
+Since these binaries can be in several different places, the script creates a container with symbolic links, where the binaries (their links, actually) are arranged as MySQL Sandbox expects them to be.
+
+To use this version, change directory to a place where you want to store this symbolic links container, and invoke
+
+  make_sandbox_from_installed X.X.XX [options]
+
+where X.X.XX is the version number. You can then pass any options accepted by make_sandbox.
+
 =head2 Defaults and shortcuts
 
 If you use sandboxes often, instead of pointing to a tarball you can set a directory containing expanded tarballs.
@@ -527,7 +538,7 @@ e.g.
 
   $HOME/opt/mysql/5.0.64 
   /opt/mysql/5.1.24
-    
+
 If you have such an organization, then you can invoke every sandbox script with this abridged syntax:
 
   make_sandbox 5.0.64
@@ -581,6 +592,11 @@ some handy scripts to manage your server easily and in isolation.
 =item stop
 
 "./start", "./restart", and "./stop" do what their name suggests.
+C<start> and C<restart> accept parameters that are eventually passed to the server. e.g.:
+
+  ./start --skip-innodb
+
+  ./restart --event-scheduler=disabled
 
 =item use
 
@@ -595,12 +611,16 @@ some handy scripts to manage your server easily and in isolation.
 On a replication sandbox, you have the same commands, with a "_all"
 suffix, meaning that you propagate the command to all the members.
 Then you have "./m" as a shortcut to use the master, "./s1" and "./s2"
-to access the slaves (and "s3", "s4" ... if you define more)
+to access the slaves (and "s3", "s4" ... if you define more).
+
+In group sandboxes without a master slave relationship (circular replication and multiple sandboxes) the nodes can be accessed by ./n1, ./n2, ./n3, and so on.
 
 =over 3
 
 =item start_all
 
+=item restart_all
+
 =item stop_all
 
 =item use_all
@@ -641,6 +661,78 @@ figure, to avoid clashing with single installations.
  |  3310  | /tmp/mysql_sandbox3310.sock |
  +--------+-----------------------------+
 
+=head2 Searching for free ports
+
+MySQL Sandbox uses a fairly reasonable system of default ports that 
+guarantees the usage of unused ports most of the times. 
+If you are creating many sandbozes, however, especially if you want
+several sandboxes using the same versions, collisions may happen.
+In these cases, you may ask for a port check before installing, thus 
+making sure that your sandbox is really not conflicting with anything.
+
+=head3 Single sandbox port checking
+
+The default behavior when asking to install a sandbox over an existing 
+one is to abort. If you specify the C<--force> option, the old sandbox
+will be overwritten.
+Instead, using the C<--check_port> option, MySQL Sandbox searches for the
+first available unused port, and uses it. It will also create a non 
+conflicting data directory. For example
+
+ make_sandbox 5.0.79
+ # creates a sandbox with port 5079 under $SANDBOX_HOME/msb_5_0_79
+
+A further call to the same command will be aborted unless you specify 
+either C<--force> or C<--check_port>.
+
+ make_sandbox 5.0.79 --force
+ # Creates a sandbox with port 5079 under $SANDBOX_HOME/msb_5_0_79
+ # The contents of the previous data directory are removed.
+
+ make_sandbox 5.0.79 --check_port
+ # Creates a sandbox with port 5080 under $SANDBOX_HOME/msb_5_0_79_a
+
+ make_sandbox 5.0.79 --check_port
+ # Creates a sandbox with port 5081 under $SANDBOX_HOME/msb_5_0_79_b
+
+Notice that this option is disabled when you use a group sandbox (replication or multiple). Even if you set NODE_OPTIONS=--check_port, it won't be used, because every group sandbox invokes make_sandbox with the --no_check_port option.
+
+=head3 Multiple sandbox port checking
+
+When you create a multiple sandbox (make_replication_sandbox, 
+make_multiple_sandbox, make_multiple_custom_sandbox) the default behavior
+is to overwrite the existing sandbox without asking for confirmation. 
+The rationale is that a multiple sandbox is definitely more likely to be a 
+created only for testing purposes, and overwriting it should not be a problem.
+If you want to avoid overwriting, you can specify a different group name
+(C<--replication_directory> C<--group_directory>), but this will use the
+same base port number, unless you specify C<--check_base_port>.
+
+ make_replication_sandbox 5.0.79
+ # Creates a replication directory under $SANDBOX_HOME/rsandbox_5_0_79
+ # The default base_port is 7000
+
+ make_replication_sandbox 5.0.79
+ # Creates a replication directory under $SANDBOX_HOME/rsandbox_5_0_79
+ # overwriting the previous one. The default base port is still 7000
+
+ # WRONG
+ make_replication_sandbox --check_base_port 5.0.79
+ # Creates a replication directory under $SANDBOX_HOME/rsandbox_5_0_79
+ # overwriting the previous one. 
+
+ # WRONG
+ make_replication_sandbox --replication_directory=newdir 5.0.79
+ # Created a replication directory under $SANDBOX_HOME/newdir.
+ # The previous one is preserved, but the new sandbox does not start
+ # because of port conflict.
+
+ # RIGHT
+ make_replication_sandbox --replication_directory=newwdir \
+    --check_base_port 5.0.79
+ # Creates a replication directory under $SANDBOX_HOME/newdir
+ # The previous one is preserved. No conflicts happen
+
 =head2 Environment variables
 
 All programs in the Sandbox suite recognize and use the following variables:
@@ -869,7 +961,9 @@ You can override this option by setting the TEST_VERSION environment variable.
 
 Starting with version 2.0.99, you can define your own tests, and run them by
 
-  $ test_sandbox --tests=user --user_test=file_name
+  $ test_sandbox --user_test=file_name
+
+=head3 simplified test script
 
 Inside your test file, you can define test actions.
 There are two kind of tests: shell and sql the test type is defined by a keyword followed by a colon.
@@ -905,6 +999,60 @@ will expand to:
 
   command = /home/sb/tests/msb_5_1_30/stop
 
+=head3 Perl based test scripts
+
+In addition to the internal script language, you can also define perl scripts, which will be able to use the $sandbox_home global variable and to call routines defined inside test_sandbox. (see list below)
+To be identified as a Perl script, the user defined test must have the extension ".sb.pl"
+
+=over 3
+
+=item ok_shell()
+
+The C<ok_shell> function requires a hash reference containing the following labels:
+A 'command', which is passed to a shell.
+The 'expected' label is a string that you expect to find within the shell output.
+If you don't expect anything, you can just say "expected = OK", meaning that you will
+be satisfied with a ZERO exit code reported by the operating system.
+The 'msg' is the description of the test that is shown to you when the test runs.
+
+  ok_shell($hashref)
+  ok_shell({
+        command  => 'make_sandbox 5.1.30 --no_confirm',
+        expected => 'sandbox server started',
+        msg      => 'sandbox creation',
+        })
+
+=item ok_sql()  
+
+The C<ok_sql> function requires a hashref containing the following labels:
+A 'path', which is the place where the test engine expects to find a 'use' script.
+The 'query' is passed to the above mentioned script and the output is captured for further processing.
+The 'expected' parameter is a string that you want to find in the query output.
+The 'msg' parameter is like the one used with the ok_exec function.
+
+
+=item get_bare_version()
+
+This function accepts one parameter, which can be either a MySQL tarball name or a version, and returns the bare version found in the input string.
+If called in list mode, it returns also a normalized version string with dots replaced by underscores.
+
+    my $version = get_bare_version('5.1.30'); 
+    # returns '5.1.30'
+
+    my $version = get_bare_version('mysql-5.1.30-OS.tar.gz'); 
+    # returns '5.1.30'
+
+    my ($version,$dir_name) = get_bare_version('mysql-5.1.30-OS.tar.gz'); 
+    # returns ('5.1.30', '5_1_30')
+
+=item ok
+
+This is a low level function, similar to the one provided by Test::More. You should not need to call this one directly, unless you want to fine tuning a test.
+
+See the test script t/start_restart_arguments.sb.pl as an example
+
+=back
+
 =head1 REQUIREMENTS
 
 To use this package you need at least the following:
