@@ -10,7 +10,7 @@ our @ISA = qw/Exporter/;
 our @EXPORT_OK = qw( scripts_in_code);
 our @EXPORT = @EXPORT_OK;
 
-our $VERSION="2.0.99";
+our $VERSION="3.0.00";
 
 our @MANIFEST = (
 'clear.sh',
@@ -300,6 +300,16 @@ my %parse_options_replication = (
                                             '(Default: 11000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
+
 
     how_many_slaves      => {
                                 value => 2,               
@@ -403,6 +413,15 @@ my %parse_options_many = (
                                             '(Default: 7000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
 
     how_many_nodes      => {
                                 value => 3,               
@@ -494,6 +513,15 @@ my %parse_options_custom_many = (
                                             '(Default: 5000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
 
     interactive           => {
                                 value => 0,                  
@@ -654,7 +682,7 @@ my %parse_options_sbtool = (
     },
 );
 
-
+# --- START SCRIPTS IN CODE ---
 
 my %scripts_in_code = (
     'start.sh' => <<'START_SCRIPT',
@@ -1010,14 +1038,15 @@ use MySQL::Sandbox;
 
 my $DEBUG = $MySQL::Sandbox::DEBUG;
 
+my $action_list = 'use|start|stop|clear|restart|send_kill';
 my $action = shift
-    or die "action required {use|start|stop|clear}\n";
-$action =~/^(use|start|stop|clear|send_kill)$/ 
-    or die "action must be one of {use|start|stop|clear|send_kill}\n";
+    or die "action required {$action_list}\n";
+$action =~/^($action_list)$/ 
+    or die "action must be one of {$action_list}\n";
 my $sandboxdir = $0;
 $sandboxdir =~ s{[^/]+$}{};
 $sandboxdir =~ s{/$}{};
-my $command = shift;
+my $command = $ARGV[0];
 if ($action eq 'use' and !$command) {
     die "action 'use' requires a command\n";
 }
@@ -1038,11 +1067,11 @@ for my $dir (@dirs) {
         } 
         elsif ( -x "$dir/${action}_all") {
             print "-- executing  $dir/${action}_all\n" if $DEBUG;
-            system("$dir/${action}_all")
+            system("$dir/${action}_all", @ARGV)
         }
         elsif ( -x "$dir/$action") {
             print "-- executing  $dir/$action\n" if $DEBUG;
-            system("$dir/$action")
+            system("$dir/$action", @ARGV)
         }
     }
 }
@@ -1051,6 +1080,8 @@ SANDBOX_ACTION_SCRIPT
 
 );
 
+# --- END SCRIPTS IN CODE ---
+
 my $license_text = <<'LICENSE';
 #    The MySQL Sandbox
 #    Copyright (C) 2006,2007,2008,2009 Giuseppe Maxia
@@ -1099,4 +1130,39 @@ sub scripts_in_code {
 }
 
 1;
+__END__
+=head1 NAME
+
+MySQL::Sandbox::Scripts - Script templates for MySQL Sandbox
+
+=head1 PURPOSE
+
+This package is a collection of data and scripts templates for MySQL::Sandbox.
+It is not supposed to be used stand alone. It is an ancillary package.
+For a reference manual, see L<MySQL::Sandbox>. For a cookbook, see L<MySQL::Sandbox::Recipes>.
+
+
+=head1 COPYRIGHT
+
+Version 3.0
+
+Copyright (C) 2006,2007,2008,2009  Giuseppe Maxia
+
+Home Page  http://launchpad.net/mysql-sandbox/
+
+=head1 LEGAL NOTICE
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
 
